@@ -2,6 +2,8 @@
 /*
  * Copyright (c) 2014-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -22,23 +24,6 @@
 #define min(a,b) ((a) < (b) ? (a) : (b))
 #define ALIGNMENT_MASK(x) ((x) ? (x) - 1 : 0)
 
-/* helper functions */
-static inline void ompi_osc_rdma_cleanup_rdma (ompi_osc_rdma_sync_t *sync, ompi_osc_rdma_frag_t *frag,
-                                               mca_btl_base_registration_handle_t *handle, ompi_osc_rdma_request_t *request)
-{
-    if (frag) {
-        ompi_osc_rdma_frag_complete (frag);
-    } else {
-        ompi_osc_rdma_deregister (sync->module, handle);
-    }
-
-    if (request) {
-        (void) OPAL_THREAD_ADD32 (&request->outstanding_requests, -1);
-    }
-
-    ompi_osc_rdma_sync_rdma_dec (sync);
-}
-
 /**
  * @brief find a remote segment associate with the memory region
  *
@@ -53,7 +38,7 @@ static inline void ompi_osc_rdma_cleanup_rdma (ompi_osc_rdma_sync_t *sync, ompi_
  * @returns OMPI_ERR_RMA_RANGE if the address range is not valid at the remote window
  * @returns other OMPI error on error
  */
-static inline int osc_rdma_get_remote_segment (ompi_osc_rdma_module_t *module, ompi_osc_rdma_peer_t *peer, OPAL_PTRDIFF_TYPE target_disp,
+static inline int osc_rdma_get_remote_segment (ompi_osc_rdma_module_t *module, ompi_osc_rdma_peer_t *peer, ptrdiff_t target_disp,
                                                size_t length, uint64_t *remote_address, mca_btl_base_registration_handle_t **remote_handle)
 {
     ompi_osc_rdma_region_t *region;
@@ -97,20 +82,20 @@ static inline int osc_rdma_get_remote_segment (ompi_osc_rdma_module_t *module, o
 /* prototypes for implementations of MPI RMA window functions. these will be called from the
  * mpi interface (ompi/mpi/c) */
 int ompi_osc_rdma_put (const void *origin_addr, int origin_count, ompi_datatype_t *origin_dt,
-                       int target, OPAL_PTRDIFF_TYPE target_disp, int target_count,
+                       int target, ptrdiff_t target_disp, int target_count,
                        ompi_datatype_t *target_dt, ompi_win_t *win);
 
 int ompi_osc_rdma_get (void *origin_addr, int origin_count, ompi_datatype_t *origin_dt,
-                       int target, OPAL_PTRDIFF_TYPE target_disp, int target_count,
+                       int target, ptrdiff_t target_disp, int target_count,
                        ompi_datatype_t *target_dt, ompi_win_t *win);
 
 int ompi_osc_rdma_rput (const void *origin_addr, int origin_count, ompi_datatype_t *origin_dt,
-                        int target, OPAL_PTRDIFF_TYPE target_disp, int target_count,
+                        int target, ptrdiff_t target_disp, int target_count,
                         ompi_datatype_t *target_dt, ompi_win_t *win,
                         ompi_request_t **request);
 
 int ompi_osc_rdma_rget (void *origin_addr, int origin_count, ompi_datatype_t *origin_dt,
-                        int target, OPAL_PTRDIFF_TYPE target_disp, int target_count,
+                        int target, ptrdiff_t target_disp, int target_count,
                         ompi_datatype_t *target_dt, ompi_win_t *win,
                         ompi_request_t **request);
 
@@ -132,4 +117,8 @@ int ompi_osc_get_data_blocking (ompi_osc_rdma_module_t *module, struct mca_btl_b
                                 uint64_t source_address, mca_btl_base_registration_handle_t *source_handle,
                                 void *data, size_t len);
 
+int ompi_osc_rdma_put_contig (ompi_osc_rdma_sync_t *sync, ompi_osc_rdma_peer_t *peer, uint64_t target_address,
+                              mca_btl_base_registration_handle_t *target_handle, void *source_buffer, size_t size,
+                              ompi_osc_rdma_request_t *request);
+
 #endif /* OMPI_OSC_RDMA_COMM_H */
