@@ -60,8 +60,11 @@ int MPI_Pack_external_size(const char datarep[], int incount,
         }
     }
 
+    OPAL_CR_ENTER_LIBRARY();
 
     rc = ompi_datatype_pack_external_size(datarep, incount,
                                           datatype, size);
+    OPAL_CR_EXIT_LIBRARY();
+
     OMPI_ERRHANDLER_RETURN(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
 }
