@@ -34,6 +34,7 @@ static opal_mutex_t lock;
 
 static int grequestx_progress(void) {
     ompi_grequest_t *request, *next;
+    int completed = 0;
 
     OPAL_THREAD_LOCK(&lock);
     if (!in_progress) {
@@ -43,18 +44,17 @@ static int grequestx_progress(void) {
             MPI_Status status;
             OPAL_THREAD_UNLOCK(&lock);
             request->greq_poll.c_poll(request->greq_state, &status);
-            if (REQUEST_COMPLETE(&request->greq_base)) {
-                OPAL_THREAD_LOCK(&lock);
-                opal_list_remove_item(&requests, &request->greq_base.super.super);
-                OPAL_THREAD_UNLOCK(&lock);
-            }
             OPAL_THREAD_LOCK(&lock);
+            if (REQUEST_COMPLETE(&request->greq_base)) {
+                opal_list_remove_item(&requests, &request->greq_base.super.super);
+                completed++;
+            }
         }
         in_progress = false;
     }
     OPAL_THREAD_UNLOCK(&lock);
 
-    return OMPI_SUCCESS;
+    return completed;
 }
 
 int ompi_grequestx_start(
