@@ -10,10 +10,11 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2017 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2013-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,6 +30,7 @@
 
 #include "opal/class/opal_object.h"
 #include "opal/class/opal_hash_table.h"
+#include "opal/util/info_subscriber.h"
 #include "ompi/errhandler/errhandler.h"
 #include "ompi/info/info.h"
 #include "ompi/communicator/communicator.h"
@@ -73,12 +75,12 @@ OMPI_DECLSPEC extern mca_base_var_enum_flag_t *ompi_win_accumulate_order;
 OMPI_DECLSPEC extern opal_pointer_array_t ompi_mpi_windows;
 
 struct ompi_win_t {
-    opal_object_t w_base;
+    opal_infosubscriber_t super;
 
     opal_mutex_t  w_lock;
 
     char w_name[MPI_MAX_OBJECT_NAME];
-
+  
     /* Group associated with this window. */
     ompi_group_t *w_group;
 
@@ -117,7 +119,7 @@ OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_win_t);
  * See ompi/communicator/communicator.h comments with struct ompi_communicator_t
  * for full explanation why we chose the following padding construct for predefines.
  */
-#define PREDEFINED_WIN_PAD (sizeof(void*) * 64)
+#define PREDEFINED_WIN_PAD 512
 
 struct ompi_predefined_win_t {
     struct ompi_win_t win;
@@ -132,13 +134,13 @@ int ompi_win_init(void);
 int ompi_win_finalize(void);
 
 int ompi_win_create(void *base, size_t size, int disp_unit,
-                    ompi_communicator_t *comm, ompi_info_t *info,
+                    ompi_communicator_t *comm, opal_info_t *info,
                     ompi_win_t **newwin);
-int ompi_win_allocate(size_t size, int disp_unit, ompi_info_t *info,
+int ompi_win_allocate(size_t size, int disp_unit, opal_info_t *info,
                       ompi_communicator_t *comm, void *baseptr, ompi_win_t **newwin);
-int ompi_win_allocate_shared(size_t size, int disp_unit, ompi_info_t *info,
+int ompi_win_allocate_shared(size_t size, int disp_unit, opal_info_t *info,
                       ompi_communicator_t *comm, void *baseptr, ompi_win_t **newwin);
-int ompi_win_create_dynamic(ompi_info_t *info, ompi_communicator_t *comm, ompi_win_t **newwin);
+int ompi_win_create_dynamic(opal_info_t *info, ompi_communicator_t *comm, ompi_win_t **newwin);
 
 int ompi_win_free(ompi_win_t *win);
 
