@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2011-2017 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2011-2018 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2011      UT-Battelle, LLC. All rights reserved.
  * Copyright (c) 2014      Research Organization for Information Science
@@ -26,6 +26,31 @@
 #include "btl_ugni_frag.h"
 
 /* helper functions */
+/**
+ * @brief Output an error message on CQ or completion error.
+ *
+ * @param[in] grc        GNI error from GNI_CqGetEvent or GNI_GetCompleted
+ * @param[in] event_data event data from GNI_CqGetEvent
+ *
+ * This is a small function to print out an error if an error
+ * was detected on a CQ event.
+ */
+int mca_btl_ugni_event_fatal_error (gni_return_t grc, gni_cq_entry_t event_data);
+
+/**
+ * @brief Attempt to re-post an rdma descriptor
+ *
+ * @param[in] rdma_desc  RDMA descriptor that failed
+ * @param[in] event_data CQ event data
+ *
+ * @returns OPAL_SUCCESS if the descriptor was re-posted
+ * @returns OPAL_ERROR otherwise
+ *
+ * This function checks if the error is recoverable and re-posts the
+ * descriptor if possible. The device lock MUST be held when this
+ * function is called.
+ */
+int mca_btl_ugni_device_handle_event_error (struct mca_btl_ugni_rdma_desc_t *rdma_desc, gni_cq_entry_t event_data);
 
 typedef struct mca_btl_ugni_smsg_send_wtag_arg_t {
     gni_ep_handle_t ep_handle;
@@ -67,19 +92,6 @@ static inline intptr_t mca_btl_ugni_smsg_release_device (mca_btl_ugni_device_t *
     return GNI_SmsgRelease (ep_handle->gni_handle);
 }
 
-static inline intptr_t mca_btl_ugni_cq_clear_device (mca_btl_ugni_device_t *device, void *arg)
-{
-    gni_cq_handle_t cq = (gni_cq_handle_t) (intptr_t) arg;
-    gni_cq_entry_t event_data;
-    int rc;
-
-    do {
-        rc = GNI_CqGetEvent (cq, &event_data);
-    } while (GNI_RC_NOT_DONE != rc);
-
-    return OPAL_SUCCESS;
-}
-
 typedef struct mca_btl_ugni_cq_get_event_args_t {
     mca_btl_ugni_cq_t *cq;
     gni_cq_entry_t *event_data;
@@ -91,10 +103,23 @@ static inline intptr_t mca_btl_ugni_cq_get_event_device (mca_btl_ugni_device_t *
     gni_return_t rc;
 
     rc = GNI_CqGetEvent (args->cq->gni_handle, args->event_data);
-    args->cq->active_operations -= GNI_RC_NOT_DONE != rc;
+    args->cq->active_operations -= (GNI_RC_NOT_DONE != rc);
     return rc;
 }
 
+static inline intptr_t mca_btl_ugni_cq_clear_device (mca_btl_ugni_device_t *device, void *arg)
+{
+    gni_cq_handle_t cq = (gni_cq_handle_t) (intptr_t) arg;
+    gni_cq_entry_t event_data;
+    int rc;
+
+    do {
+        rc = GNI_CqGetEvent (cq, &event_data);
+    } while (GNI_RC_NOT_DONE != rc);
+
+    return OPAL_SUCCESS;
+}
+
 typedef struct mca_btl_ugni_gni_cq_get_event_args_t {
     gni_cq_handle_t cq;
     gni_cq_entry_t *event_data;
@@ -107,148 +132,246 @@ static inline intptr_t mca_btl_ugni_gni_cq_get_event_device (mca_btl_ugni_device
     return GNI_CqGetEvent (args->cq, args->event_data);
 }
 
-static inline intptr_t mca_btl_ugni_post_fma_device (mca_btl_ugni_device_t *device, void *arg)
+typedef struct mca_btl_ugni_cq_get_completed_desc_arg_t {
+    mca_btl_ugni_cq_t *cq;
+    mca_btl_ugni_post_descriptor_t *post_desc;
+    int count;
+} mca_btl_ugni_cq_get_completed_desc_arg_t;
+
+__opal_attribute_always_inline__
+static inline int _mca_btl_ugni_repost_rdma_desc_device (mca_btl_ugni_device_t *device, mca_btl_ugni_rdma_desc_t *rdma_desc)
 {
-    mca_btl_ugni_post_descriptor_t *desc = (mca_btl_ugni_post_descriptor_t *) arg;
-    bool ep_handle_allocated = false;
+    mca_btl_ugni_post_descriptor_t *post_desc = &rdma_desc->btl_ugni_desc;
     int rc;
 
-    if (NULL == desc->ep_handle) {
-        desc->ep_handle = mca_btl_ugni_ep_get_rdma (desc->endpoint, device);
-        if (OPAL_UNLIKELY(NULL == desc->ep_handle)) {
-            return OPAL_ERR_TEMP_OUT_OF_RESOURCE;
-        }
-        ep_handle_allocated = true;
-    }
-
-    BTL_VERBOSE(("Posting FMA descriptor %p with op_type %d, amo %d, ep_handle %p, remote_addr 0x%lx, "
-                 "length %lu", (void*)desc, desc->desc.type, desc->desc.amo_cmd, (void*)desc->ep_handle,
-                 desc->desc.remote_addr, desc->desc.length));
-
-    rc = GNI_PostFma (desc->ep_handle->gni_handle, &desc->desc);
-    if (OPAL_UNLIKELY(GNI_RC_SUCCESS != rc)) {
-        if (ep_handle_allocated) {
-            /* only return the endpoint handle if we allocated it. if we didn't allocate the
-             * handle this call was likely made from repost() */
-            mca_btl_ugni_ep_return_rdma (desc->ep_handle);
-            desc->ep_handle = NULL;
-        }
+    if (post_desc->use_bte) {
+        rc = GNI_PostRdma (rdma_desc->gni_handle, &post_desc->gni_desc);
     } else {
-        ++device->dev_rdma_local_cq.active_operations;
+        rc = GNI_PostFma (rdma_desc->gni_handle, &post_desc->gni_desc);
     }
 
     return mca_btl_rc_ugni_to_opal (rc);
 }
 
-static inline intptr_t mca_btl_ugni_post_rdma_device (mca_btl_ugni_device_t *device, void *arg)
+static inline intptr_t _mca_btl_ugni_cq_get_completed_desc_device (mca_btl_ugni_device_t *device, mca_btl_ugni_cq_t *cq,
+                                                                   mca_btl_ugni_post_descriptor_t *post_desc,
+                                                                   const int count, bool block)
 {
-    mca_btl_ugni_post_descriptor_t *desc = (mca_btl_ugni_post_descriptor_t *) arg;
-    bool ep_handle_allocated = false;
+    mca_btl_ugni_rdma_desc_t *rdma_desc;
+    gni_post_descriptor_t *desc;
+    gni_cq_entry_t event_data;
+    int rc, desc_index = 0;
+
+    for (desc_index = 0 ; desc_index < count && cq->active_operations ; ) {
+        int desc_rc = OPAL_SUCCESS;
+
+        rc = GNI_CqGetEvent (cq->gni_handle, &event_data);
+        if (GNI_RC_NOT_DONE == rc) {
+	    if (block) {
+		/* try again */
+		continue;
+	    }
+            break;
+        }
+
+	block = false;
+
+        rc = GNI_GetCompleted (cq->gni_handle, event_data, &desc);
+        if (OPAL_UNLIKELY(GNI_RC_SUCCESS != rc && GNI_RC_TRANSACTION_ERROR != rc)) {
+            return mca_btl_ugni_event_fatal_error (rc, event_data);
+        }
+
+        rdma_desc = MCA_BTL_UGNI_GNI_DESC_TO_RDMA_DESC(desc);
+
+        if (OPAL_UNLIKELY(!GNI_CQ_STATUS_OK(event_data))) {
+            desc_rc = mca_btl_ugni_device_handle_event_error (rdma_desc, event_data);
+            if (OPAL_LIKELY(OPAL_SUCCESS == desc_rc)) {
+                /* descriptor was re-posted */
+                continue;
+            }
+        }
+
+        /* copy back the descriptor only if additional processing is needed. in this case more processing
+         * is needed if a user callback is specified or the bte was in use. */
+        if (rdma_desc->btl_ugni_desc.cbfunc || rdma_desc->btl_ugni_desc.use_bte || OPAL_SUCCESS != desc_rc) {
+            post_desc[desc_index] = rdma_desc->btl_ugni_desc;
+            post_desc[desc_index++].rc = desc_rc;
+        }
+
+        /* return the descriptor while we have the lock. this is done so we can avoid using the
+         * free list atomics (as both push and pop are done with the lock) */
+        mca_btl_ugni_return_rdma_desc (rdma_desc);
+        --cq->active_operations;
+    }
+
+    return desc_index;
+}
+
+static inline intptr_t mca_btl_ugni_cq_get_completed_desc_device (mca_btl_ugni_device_t *device, void *arg0)
+{
+    mca_btl_ugni_cq_get_completed_desc_arg_t *args = (mca_btl_ugni_cq_get_completed_desc_arg_t *) arg0;
+
+    return _mca_btl_ugni_cq_get_completed_desc_device (device, args->cq, args->post_desc, args->count, false);
+}
+
+/* NTH: When posting FMA or RDMA descriptors it makes sense to try and clear out a completion
+ * event after posting the descriptor. This probably gives us a couple of things:
+ *   1) Good locality on the associated data structures (especially with FMA which may
+ *      complete fairly quickly).
+ *   2) Since we are already holding the lock it could mean fewer attempts to
+ *      lock the device over the course of the program.
+ *
+ * As far as I can tell there is not reason to try and clear out more than a couple
+ * completiong events. The code has been written to allow us to easily modify the
+ * number reaped if we determine that there is a benefit to clearing a different
+ * number of events. */
+
+/**
+ * @brief Number of events to clear after posting a descriptor
+ */
+#define MCA_BTL_UGNI_DEVICE_REAP_COUNT 4
+
+struct mca_btl_ugni_post_device_args_t {
+    mca_btl_ugni_post_descriptor_t *desc;
+    mca_btl_ugni_device_t *device;
+    int count;
+    mca_btl_ugni_post_descriptor_t completed[MCA_BTL_UGNI_DEVICE_REAP_COUNT];
+};
+
+static inline mca_btl_ugni_rdma_desc_t *
+mca_btl_ugni_get_rdma_desc_device (mca_btl_ugni_device_t *device, struct mca_btl_ugni_post_device_args_t *args, bool use_bte)
+{
+    mca_btl_ugni_post_descriptor_t *desc = args->desc;
+    mca_btl_ugni_rdma_desc_t *rdma_desc;
+
+    args->device = device;
+    args->count = 0;
+
+    do {
+        rdma_desc = mca_btl_ugni_alloc_rdma_desc (device, desc, use_bte);
+	if (OPAL_LIKELY(NULL != rdma_desc)) {
+	    return rdma_desc;
+	}
+
+        if (OPAL_LIKELY(NULL == rdma_desc && !args->count)) {
+	    args->count = _mca_btl_ugni_cq_get_completed_desc_device (device, &device->dev_rdma_local_cq,
+								      args->completed, MCA_BTL_UGNI_DEVICE_REAP_COUNT,
+								      true);
+	    continue;
+        }
+
+	return NULL;
+    } while (1);
+}
+
+
+static inline intptr_t mca_btl_ugni_post_fma_device (mca_btl_ugni_device_t *device, void *arg)
+{
+    struct mca_btl_ugni_post_device_args_t *args = (struct mca_btl_ugni_post_device_args_t *) arg;
+    mca_btl_ugni_rdma_desc_t *rdma_desc;
     int rc;
 
-    if (NULL == desc->ep_handle) {
-        desc->ep_handle = mca_btl_ugni_ep_get_rdma (desc->endpoint, device);
-        if (OPAL_UNLIKELY(NULL == desc->ep_handle)) {
-            return OPAL_ERR_TEMP_OUT_OF_RESOURCE;
-        }
-        ep_handle_allocated = true;
+    rdma_desc = mca_btl_ugni_get_rdma_desc_device (device, args, false);
+    if (OPAL_UNLIKELY(NULL == rdma_desc)) {
+	return OPAL_ERR_TEMP_OUT_OF_RESOURCE;
+    }
+
+    BTL_VERBOSE(("Posting FMA descriptor %p with op_type %d, amo %d, remote_addr 0x%lx, "
+                 "length %lu", (void*)rdma_desc, rdma_desc->btl_ugni_desc.gni_desc.type, rdma_desc->btl_ugni_desc.gni_desc.amo_cmd,
+                 rdma_desc->btl_ugni_desc.gni_desc.remote_addr, rdma_desc->btl_ugni_desc.gni_desc.length));
+
+    rc = GNI_PostFma (rdma_desc->gni_handle, &rdma_desc->btl_ugni_desc.gni_desc);
+    if (OPAL_UNLIKELY(GNI_RC_SUCCESS != rc)) {
+        mca_btl_ugni_return_rdma_desc (rdma_desc);
+        return mca_btl_rc_ugni_to_opal (rc);
+    }
+
+    ++device->dev_rdma_local_cq.active_operations;
+
+    /* to improve bandwidth and latency it is ideal for all posting threads to also reap completions from
+     * the rdma completion queue. there are two optimizations here. 1) for bandwidth we only want to
+     * reap what is available now so more messages can be posted quickly, and 2) for latency (single
+     * put/get before flushing) we want to ensure the operation is complete. To some degree this is
+     * gaming the benchmark but it may benefit some application communication patterns without really
+     * hurting others (in theory). */
+    if (opal_using_threads ()) {
+	int count = args->count;
+	args->count += _mca_btl_ugni_cq_get_completed_desc_device (device, &device->dev_rdma_local_cq,
+								   args->completed + count,
+								   MCA_BTL_UGNI_DEVICE_REAP_COUNT - count,
+								   device->flushed);
+	device->flushed = false;
+    }
+
+    return OPAL_SUCCESS;
+}
+
+static inline intptr_t mca_btl_ugni_post_rdma_device (mca_btl_ugni_device_t *device, void *arg)
+{
+    struct mca_btl_ugni_post_device_args_t *args = (struct mca_btl_ugni_post_device_args_t *) arg;
+    mca_btl_ugni_rdma_desc_t *rdma_desc;
+    int rc;
+
+    rdma_desc = mca_btl_ugni_get_rdma_desc_device (device, args, true);
+    if (OPAL_UNLIKELY(NULL == rdma_desc)) {
+	return OPAL_ERR_TEMP_OUT_OF_RESOURCE;
     }
 
     /* pick the appropriate CQ */
-    desc->cq = mca_btl_ugni_component.progress_thread_enabled ? &device->dev_rdma_local_irq_cq :
+    rdma_desc->btl_ugni_desc.cq = mca_btl_ugni_component.progress_thread_enabled ? &device->dev_rdma_local_irq_cq :
         &device->dev_rdma_local_cq;
 
-    desc->desc.src_cq_hndl = desc->cq->gni_handle;
+    BTL_VERBOSE(("Posting RDMA descriptor %p with op_type %d, amo %d, remote_addr 0x%lx, "
+                 "length %lu", (void*)rdma_desc, rdma_desc->btl_ugni_desc.gni_desc.type, rdma_desc->btl_ugni_desc.gni_desc.amo_cmd,
+                 rdma_desc->btl_ugni_desc.gni_desc.remote_addr, rdma_desc->btl_ugni_desc.gni_desc.length));
 
-    BTL_VERBOSE(("Posting RDMA descriptor %p with op_type %d, ep_handle %p, remote_addr 0x%lx, "
-                 "length %lu", (void*)desc, desc->desc.type, (void*)desc->ep_handle, desc->desc.remote_addr,
-                 desc->desc.length));
-
-    rc = GNI_PostRdma (desc->ep_handle->gni_handle, &desc->desc);
+    rc = GNI_PostRdma (rdma_desc->gni_handle, &rdma_desc->btl_ugni_desc.gni_desc);
     if (OPAL_UNLIKELY(GNI_RC_SUCCESS != rc)) {
-        if (ep_handle_allocated) {
-            /* only return the endpoint handle if we allocated it. if we didn't allocate the
-             * handle this call was likely made from repost() */
-            mca_btl_ugni_ep_return_rdma (desc->ep_handle);
-            desc->ep_handle = NULL;
-        }
-    } else {
-        ++desc->cq->active_operations;
+        mca_btl_ugni_return_rdma_desc (rdma_desc);
+        return mca_btl_rc_ugni_to_opal (rc);
     }
 
-    return mca_btl_rc_ugni_to_opal (rc);
+    ++rdma_desc->btl_ugni_desc.cq->active_operations;
+
+    /* to improve bandwidth and latency it is ideal for all posting threads to also reap completions from
+     * the rdma completion queue. there are two optimizations here. 1) for bandwidth we only want to
+     * reap what is available now so more messages can be posted quickly, and 2) for latency (single
+     * put/get before flushing) we want to ensure the operation is complete. To some degree this is
+     * gaming the benchmark but it may benefit some application communication patterns without really
+     * hurting others (in theory). */
+    if (opal_using_threads ()) {
+	int count = args->count;
+	args->count += _mca_btl_ugni_cq_get_completed_desc_device (device, &device->dev_rdma_local_cq,
+								   args->completed + count,
+								   MCA_BTL_UGNI_DEVICE_REAP_COUNT - count,
+								   device->flushed);
+	device->flushed = false;
+    }
+
+    return OPAL_SUCCESS;
 }
 
 static inline intptr_t mca_btl_ugni_post_cqwrite_device (mca_btl_ugni_device_t *device, void *arg)
 {
     mca_btl_ugni_post_descriptor_t *desc = (mca_btl_ugni_post_descriptor_t *) arg;
+    mca_btl_ugni_rdma_desc_t *rdma_desc;
     int rc;
 
-    desc->ep_handle = mca_btl_ugni_ep_get_rdma (desc->endpoint, device);
-    if (OPAL_UNLIKELY(NULL == desc->ep_handle)) {
+    desc->gni_desc.src_cq_hndl = device->dev_rdma_local_cq.gni_handle;
+
+    rdma_desc = mca_btl_ugni_alloc_rdma_desc (device, desc, false);
+    if (OPAL_UNLIKELY(NULL == rdma_desc)) {
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
 
-    desc->desc.src_cq_hndl = device->dev_rdma_local_cq.gni_handle;
-
-    rc = GNI_PostCqWrite (desc->ep_handle->gni_handle, &desc->desc);
+    rc = GNI_PostCqWrite (rdma_desc->gni_handle, &rdma_desc->btl_ugni_desc.gni_desc);
     if (OPAL_UNLIKELY(GNI_RC_SUCCESS != rc)) {
-        mca_btl_ugni_ep_return_rdma (desc->ep_handle);
-        desc->ep_handle = NULL;
+        mca_btl_ugni_return_rdma_desc (rdma_desc);
     }
 
     return mca_btl_rc_ugni_to_opal (rc);
 }
 
-typedef struct mca_btl_ugni_cq_get_completed_desc_arg_t {
-    mca_btl_ugni_cq_t *cq;
-    gni_cq_entry_t *event_data;
-    mca_btl_ugni_post_descriptor_t **post_desc;
-    int count;
-} mca_btl_ugni_cq_get_completed_desc_arg_t;
-
-static inline intptr_t mca_btl_ugni_cq_get_completed_desc_device (mca_btl_ugni_device_t *device, void *arg0)
-{
-    mca_btl_ugni_cq_get_completed_desc_arg_t *args = (mca_btl_ugni_cq_get_completed_desc_arg_t *) arg0;
-    mca_btl_ugni_cq_t *cq = args->cq;
-    gni_post_descriptor_t *desc;
-    int rc;
-
-    for (int i = 0 ; i < args->count ; ++i) {
-        rc = GNI_CqGetEvent (cq->gni_handle, args->event_data + i);
-        if (GNI_RC_NOT_DONE == rc) {
-            return i;
-        }
-
-        if (OPAL_UNLIKELY((GNI_RC_SUCCESS != rc && !args->event_data[i]) || GNI_CQ_OVERRUN(args->event_data[i]))) {
-            /* TODO -- need to handle overrun -- how do we do this without an event?
-               will the event eventually come back? Ask Cray */
-            BTL_ERROR(("unhandled post error! ugni rc = %d %s", rc, gni_err_str[rc]));
-
-            return mca_btl_rc_ugni_to_opal (rc);
-        }
-
-        rc = GNI_GetCompleted (cq->gni_handle, args->event_data[i], &desc);
-        if (OPAL_UNLIKELY(GNI_RC_SUCCESS != rc && GNI_RC_TRANSACTION_ERROR != rc)) {
-            BTL_ERROR(("Error in GNI_GetComplete %s", gni_err_str[rc]));
-            return mca_btl_rc_ugni_to_opal (rc);
-        }
-
-        args->post_desc[i] = MCA_BTL_UGNI_DESC_TO_PDESC(desc);
-        /* return the endpoint handle while we have the lock. see the explanation in
-         * the documentation for mca_btl_ugni_ep_return_rdma() */
-        if (OPAL_LIKELY(GNI_CQ_STATUS_OK(args->event_data[i]))) {
-            /* the operation completed successfully. return the endpoint handle now. otherwise
-             * we may still need the endpoint handle to start the repost(). */
-            mca_btl_ugni_ep_return_rdma (args->post_desc[i]->ep_handle);
-            args->post_desc[i]->ep_handle = NULL;
-        }
-        --cq->active_operations;
-    }
-
-    return args->count;
-}
-
 typedef struct mca_btl_ugni_get_datagram_args_t {
     mca_btl_ugni_module_t *ugni_module;
     gni_ep_handle_t *handle;
@@ -275,7 +398,7 @@ static inline intptr_t mca_btl_ugni_get_datagram_device (mca_btl_ugni_device_t *
 
     if ((datagram_id & MCA_BTL_UGNI_DATAGRAM_MASK) == MCA_BTL_UGNI_CONNECT_DIRECTED_ID) {
         *(args->ep) = (mca_btl_base_endpoint_t *) opal_pointer_array_get_item (&args->ugni_module->endpoints, data);
-        *(args->handle) = (*args->ep)->smsg_ep_handle->gni_handle;
+        *(args->handle) = (*args->ep)->smsg_ep_handle.gni_handle;
     } else {
         *(args->handle) = args->ugni_module->wildcard_ep;
     }
@@ -336,11 +459,11 @@ static intptr_t mca_btl_ugni_dereg_mem_device (mca_btl_ugni_device_t *device, vo
 static inline int mca_btl_ugni_endpoint_smsg_send_wtag (mca_btl_base_endpoint_t *endpoint, void *hdr, size_t hdr_len,
                                                         void *payload, size_t payload_len, uint32_t msg_id, int tag)
 {
-    mca_btl_ugni_smsg_send_wtag_arg_t args = {.ep_handle = endpoint->smsg_ep_handle->gni_handle,
+    mca_btl_ugni_smsg_send_wtag_arg_t args = {.ep_handle = endpoint->smsg_ep_handle.gni_handle,
                                               .hdr = hdr, .hdr_len = hdr_len, .payload = payload,
                                               .payload_len = payload_len, .msg_id = msg_id,
                                               .tag = tag};
-    mca_btl_ugni_device_t *device = endpoint->smsg_ep_handle->device;
+    mca_btl_ugni_device_t *device = endpoint->smsg_ep_handle.device;
     return (int) mca_btl_ugni_device_serialize (device, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_smsg_send_wtag_device, &args);
 }
 
@@ -367,6 +490,10 @@ static inline void mca_btl_ugni_cq_clear (mca_btl_ugni_device_t *device, gni_cq_
 static inline int mca_btl_ugni_cq_get_event (mca_btl_ugni_device_t *device, mca_btl_ugni_cq_t *cq, gni_cq_entry_t *event_data)
 {
     mca_btl_ugni_cq_get_event_args_t args = {.cq = cq, .event_data = event_data};
+    /* NTH: normally there would be a check for any outstanding CQ operations but there seems
+     * to be a reason to check the local SMSG completion queue anyway. since this function
+     * only handled the SMSG local completion queue not checking here should be fine and
+     * should not impact performance. */
     return (int) mca_btl_ugni_device_serialize (device, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_cq_get_event_device, &args);
 }
 
@@ -376,18 +503,34 @@ static inline int mca_btl_ugni_gni_cq_get_event (mca_btl_ugni_device_t *device,
     return (int) mca_btl_ugni_device_serialize (device, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_gni_cq_get_event_device, &args);
 }
 
-static inline int mca_btl_ugni_endpoint_post_fma (mca_btl_ugni_endpoint_t *endpoint, mca_btl_ugni_post_descriptor_t *desc)
+__opal_attribute_always_inline__
+static inline int mca_btl_ugni_endpoint_post (mca_btl_ugni_endpoint_t *endpoint, mca_btl_ugni_post_descriptor_t *desc,
+                                              mca_btl_ugni_device_serialize_fn_t post_fn)
 {
+    struct mca_btl_ugni_post_device_args_t args = {.desc = desc};
     mca_btl_ugni_module_t *ugni_module = mca_btl_ugni_ep_btl (endpoint);
-    mca_btl_ugni_device_t *device = desc->ep_handle ? desc->ep_handle->device : mca_btl_ugni_ep_get_device (ugni_module);
-    return (int) mca_btl_ugni_device_serialize (device, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_post_fma_device, desc);
+    int rc;
+
+    /* use serialize_any as it is responsible for binding devices to threads (if enabled). this generally
+     * gives better performance as it reduces contention on any individual device. */
+    rc = mca_btl_ugni_device_serialize_any (ugni_module, post_fn, &args);
+    if (args.count) {
+        mca_btl_ugni_handle_rdma_completions (ugni_module, args.device, args.completed, args.count);
+    }
+
+    return rc;
 }
 
+__opal_attribute_always_inline__
+static inline int mca_btl_ugni_endpoint_post_fma (mca_btl_ugni_endpoint_t *endpoint, mca_btl_ugni_post_descriptor_t *desc)
+{
+    return mca_btl_ugni_endpoint_post (endpoint, desc, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_post_fma_device);
+}
+
+__opal_attribute_always_inline__
 static inline int mca_btl_ugni_endpoint_post_rdma (mca_btl_ugni_endpoint_t *endpoint, mca_btl_ugni_post_descriptor_t *desc)
 {
-    mca_btl_ugni_module_t *ugni_module = mca_btl_ugni_ep_btl (endpoint);
-    mca_btl_ugni_device_t *device = desc->ep_handle ? desc->ep_handle->device : mca_btl_ugni_ep_get_device (ugni_module);
-    return (int) mca_btl_ugni_device_serialize (device, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_post_rdma_device, desc);
+    return mca_btl_ugni_endpoint_post (endpoint, desc, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_post_rdma_device);
 }
 
 static inline int mca_btl_ugni_endpoint_post_cqwrite (mca_btl_ugni_endpoint_t *endpoint, mca_btl_ugni_post_descriptor_t *desc)
@@ -397,11 +540,16 @@ static inline int mca_btl_ugni_endpoint_post_cqwrite (mca_btl_ugni_endpoint_t *e
     return (int) mca_btl_ugni_device_serialize (device, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_post_cqwrite_device, desc);
 }
 
+__opal_attribute_always_inline__
 static inline int mca_btl_ugni_cq_get_completed_desc (mca_btl_ugni_device_t *device, mca_btl_ugni_cq_t *cq,
-                                                      gni_cq_entry_t *event_data, mca_btl_ugni_post_descriptor_t **post_desc,
+                                                      mca_btl_ugni_post_descriptor_t *post_desc,
                                                       int count)
 {
-    mca_btl_ugni_cq_get_completed_desc_arg_t args = {.cq = cq, .event_data = event_data, .post_desc = post_desc, .count = count};
+    mca_btl_ugni_cq_get_completed_desc_arg_t args = {.cq = cq, .post_desc = post_desc, .count = count};
+    if (0 == cq->active_operations) {
+        return 0;
+    }
+
     return (int) mca_btl_ugni_device_serialize (device, (mca_btl_ugni_device_serialize_fn_t) mca_btl_ugni_cq_get_completed_desc_device, &args);
 }
 
