@@ -2,10 +2,10 @@
 /*
  * Copyright (c) 2007      The Trustees of Indiana University.
  *                         All rights reserved.
- * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011-2013 Los Alamos National Security, LLC. All
+ * Copyright (c) 2011-2016 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011-2017 Los Alamos National Security, LLC. All
  *                         rights reserved.
- * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2018 Intel, Inc. All rights reserved.
  * Copyright (c) 2014-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -36,7 +36,7 @@
 #include "opal/mca/pmix/base/pmix_base_hash.h"
 #include "pmix_s2.h"
 
-static int s2_init(void);
+static int s2_init(opal_list_t *ilist);
 static int s2_fini(void);
 static int s2_initialized(void);
 static int s2_abort(int flag, const char msg[],
@@ -77,8 +77,8 @@ const opal_pmix_base_module_t opal_pmix_s2_module = {
     .spawn = s2_spawn,
     .connect = s2_job_connect,
     .disconnect = s2_job_disconnect,
-    .register_errhandler = opal_pmix_base_register_handler,
-    .deregister_errhandler = opal_pmix_base_deregister_handler,
+    .register_evhandler = opal_pmix_base_register_handler,
+    .deregister_evhandler = opal_pmix_base_deregister_handler,
     .store_local = s2_store_local,
     .get_nspace = s2_get_nspace,
     .register_jobid = s2_register_jobid
@@ -126,9 +126,9 @@ static bool got_modex_data = false;
 static char* pmix_error(int pmix_err);
 #define OPAL_PMI_ERROR(pmi_err, pmi_func)                       \
     do {                                                        \
-        opal_output(0, "%s [%s:%d:%s]: %s\n",                   \
-                    pmi_func, __FILE__, __LINE__, __func__,     \
-                    pmix_error(pmi_err));                        \
+    opal_output(0, "%s [%s:%d:%s]: %s\n",                   \
+    pmi_func, __FILE__, __LINE__, __func__,     \
+    pmix_error(pmi_err));                        \
     } while(0);
 
 static int kvs_put(const char key[], const char value[])
@@ -158,7 +158,7 @@ static int kvs_get(const char key[], char value [], int maxvalue)
     return OPAL_SUCCESS;
 }
 
-static int s2_init(void)
+static int s2_init(opal_list_t *ilist)
 {
     int spawned, size, rank, appnum;
     int rc, ret = OPAL_ERROR;
@@ -172,6 +172,12 @@ static int s2_init(void)
     char **localranks;
     char *str;
     char nmtmp[64];
+    opal_process_name_t wildcard_rank;
+
+    if (0 < pmix_init_count) {
+        ++pmix_init_count;
+        return OPAL_SUCCESS;
+    }
 
     /* if we can't startup PMI, we can't be used */
     if ( PMI2_Initialized () ) {
@@ -189,6 +195,7 @@ static int s2_init(void)
     }
     if( size < 0 || rank < 0 ){
         opal_show_help("help-pmix-base.txt", "pmix2-init-returned-bad-values", true);
+        ret = OPAL_ERR_BAD_PARAM;
         goto err_exit;
     }
 
@@ -212,6 +219,7 @@ static int s2_init(void)
     if( PMI2_SUCCESS != rc ) {
         OPAL_PMI_ERROR(rc, "PMI2_Job_GetId");
         free(pmix_kvs_name);
+        ret = OPAL_ERR_BAD_PARAM;
         goto err_exit;
     }
 
@@ -222,7 +230,7 @@ static int s2_init(void)
     s2_pname.jobid = strtoul(pmix_kvs_name, &str, 10);
     s2_pname.jobid = (s2_pname.jobid << 16) & 0xffff0000;
     if (NULL != str) {
-        stepid = strtoul(str, NULL, 10);
+        stepid = strtoul(str+1, NULL, 10);
         s2_pname.jobid |= (stepid & 0x0000ffff);
     }
     s2_pname.vpid = s2_rank;
@@ -231,6 +239,10 @@ static int s2_init(void)
                         "%s pmix:s2: assigned tmp name",
                         OPAL_NAME_PRINT(s2_pname));
 
+    /* setup wildcard rank*/
+    wildcard_rank = OPAL_PROC_MY_NAME;
+    wildcard_rank.vpid = OPAL_VPID_WILDCARD;
+
     /* Slurm PMI provides the job id as an integer followed
      * by a '.', followed by essentially a stepid. The first integer
      * defines an overall job number. The second integer is the number of
@@ -240,7 +252,7 @@ static int s2_init(void)
     kv.key = strdup(OPAL_PMIX_JOBID);
     kv.type = OPAL_UINT32;
     kv.data.uint32 = s2_pname.jobid;
-    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&OPAL_PROC_MY_NAME, &kv))) {
+    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&wildcard_rank, &kv))) {
         OPAL_ERROR_LOG(ret);
         OBJ_DESTRUCT(&kv);
         goto err_exit;
@@ -252,8 +264,8 @@ static int s2_init(void)
     kv.key = strdup(OPAL_PMIX_JOB_SIZE);
     kv.type = OPAL_UINT32;
     kv.data.uint32 = size;
-    if (OPAL_SUCCESS != (rc = opal_pmix_base_store(&OPAL_PROC_MY_NAME, &kv))) {
-        OPAL_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&wildcard_rank, &kv))) {
+        OPAL_ERROR_LOG(ret);
         OBJ_DESTRUCT(&kv);
         goto err_exit;
     }
@@ -274,6 +286,7 @@ static int s2_init(void)
     rc = PMI2_Info_GetJobAttr("universeSize", buf, 16, &found);
     if( PMI2_SUCCESS != rc ) {
         OPAL_PMI_ERROR(rc, "PMI_Get_universe_size");
+        ret = OPAL_ERR_BAD_PARAM;
         goto err_exit;
     }
     /* save it */
@@ -281,8 +294,8 @@ static int s2_init(void)
     kv.key = strdup(OPAL_PMIX_UNIV_SIZE);
     kv.type = OPAL_UINT32;
     kv.data.uint32 = atoi(buf);
-    if (OPAL_SUCCESS != (rc = opal_pmix_base_store(&OPAL_PROC_MY_NAME, &kv))) {
-        OPAL_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&wildcard_rank, &kv))) {
+        OPAL_ERROR_LOG(ret);
         OBJ_DESTRUCT(&kv);
         goto err_exit;
     }
@@ -291,8 +304,8 @@ static int s2_init(void)
     OBJ_CONSTRUCT(&kv, opal_value_t);
     kv.key = strdup(OPAL_PMIX_MAX_PROCS);
     kv.type = OPAL_UINT32;
-    kv.data.uint32 = i;
-    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&OPAL_PROC_MY_NAME, &kv))) {
+    kv.data.uint32 = atoi(buf);
+    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&wildcard_rank, &kv))) {
         OPAL_ERROR_LOG(ret);
         OBJ_DESTRUCT(&kv);
         goto err_exit;
@@ -301,22 +314,23 @@ static int s2_init(void)
 
     char *pmapping = (char*)malloc(PMI2_MAX_VALLEN);
     if( pmapping == NULL ){
-        rc = OPAL_ERR_OUT_OF_RESOURCE;
-        OPAL_ERROR_LOG(rc);
-        return rc;
+        ret = OPAL_ERR_OUT_OF_RESOURCE;
+        OPAL_ERROR_LOG(ret);
+        goto err_exit;
     }
 
     rc = PMI2_Info_GetJobAttr("PMI_process_mapping", pmapping, PMI2_MAX_VALLEN, &found);
     if( !found || PMI2_SUCCESS != rc ) {
         OPAL_PMI_ERROR(rc,"PMI2_Info_GetJobAttr");
-        return OPAL_ERROR;
+        ret = OPAL_ERR_BAD_PARAM;
+        goto err_exit;
     }
 
     s2_lranks = mca_common_pmi2_parse_pmap(pmapping, s2_pname.vpid, &my_node, &s2_nlranks);
     if (NULL == s2_lranks) {
-        rc = OPAL_ERR_OUT_OF_RESOURCE;
-        OPAL_ERROR_LOG(rc);
-        return rc;
+        ret = OPAL_ERR_OUT_OF_RESOURCE;
+        OPAL_ERROR_LOG(ret);
+        goto err_exit;
     }
 
     free(pmapping);
@@ -326,8 +340,8 @@ static int s2_init(void)
     kv.key = strdup(OPAL_PMIX_LOCAL_SIZE);
     kv.type = OPAL_UINT32;
     kv.data.uint32 = s2_nlranks;
-    if (OPAL_SUCCESS != (rc = opal_pmix_base_store(&OPAL_PROC_MY_NAME, &kv))) {
-        OPAL_ERROR_LOG(rc);
+    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&wildcard_rank, &kv))) {
+        OPAL_ERROR_LOG(ret);
         OBJ_DESTRUCT(&kv);
         goto err_exit;
     }
@@ -358,7 +372,7 @@ static int s2_init(void)
         kv.key = strdup(OPAL_PMIX_LOCAL_PEERS);
         kv.type = OPAL_STRING;
         kv.data.string = str;
-        if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&OPAL_PROC_MY_NAME, &kv))) {
+        if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&wildcard_rank, &kv))) {
             OPAL_ERROR_LOG(ret);
             OBJ_DESTRUCT(&kv);
             goto err_exit;
@@ -371,7 +385,7 @@ static int s2_init(void)
     kv.key = strdup(OPAL_PMIX_LOCALLDR);
     kv.type = OPAL_UINT64;
     kv.data.uint64 = *(uint64_t*)&ldr;
-    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&OPAL_PROC_MY_NAME, &kv))) {
+    if (OPAL_SUCCESS != (ret = opal_pmix_base_store(&wildcard_rank, &kv))) {
         OPAL_ERROR_LOG(ret);
         OBJ_DESTRUCT(&kv);
         goto err_exit;
@@ -404,7 +418,7 @@ static int s2_init(void)
     ++pmix_init_count;
 
     return OPAL_SUCCESS;
- err_exit:
+err_exit:
     PMI2_Finalize();
     return ret;
 }
@@ -416,13 +430,13 @@ static int s2_fini(void) {
 
     if (0 == --pmix_init_count) {
         PMI2_Finalize();
-    }
-    if (NULL != pmix_kvs_name) {
-        free(pmix_kvs_name);
-        pmix_kvs_name = NULL;
-    }
-    if (NULL != s2_lranks) {
-        free(s2_lranks);
+        if (NULL != pmix_kvs_name) {
+            free(pmix_kvs_name);
+            pmix_kvs_name = NULL;
+        }
+        if (NULL != s2_lranks) {
+            free(s2_lranks);
+        }
     }
     return OPAL_SUCCESS;
 }
@@ -453,8 +467,8 @@ static int s2_spawn(opal_list_t *jobinfo, opal_list_t *apps, opal_jobid_t *jobid
     preput_vector_size = opal_list_get_size(preput_keyval_vector);
     rc = PMI2_Job_Spawn(count, cmds, argcs, argvs, maxprocs, info_keyval_sizes, info_keyval_vector, (int)preput_vector_size, preput_keyval_vector, jobId, jobIdSize, errors);
     if( PMI2_SUCCESS != rc ) {
-        OPAL_PMI_ERROR(rc, "PMI2_Job_Spawn");
-        return OPAL_ERROR;
+    OPAL_PMI_ERROR(rc, "PMI2_Job_Spawn");
+    return OPAL_ERROR;
     }*/
     return OPAL_ERR_NOT_IMPLEMENTED;
 }
@@ -511,8 +525,8 @@ static int s2_put(opal_pmix_scope_t scope,
     int rc;
 
     opal_output_verbose(2, opal_pmix_base_framework.framework_output,
-            "%s pmix:s2 put for key %s",
-            OPAL_NAME_PRINT(OPAL_PROC_MY_NAME), kv->key);
+                        "%s pmix:s2 put for key %s",
+                        OPAL_NAME_PRINT(OPAL_PROC_MY_NAME), kv->key);
 
     if (OPAL_SUCCESS != (rc = opal_pmix_base_store_encoded (kv->key, (void*)&kv->data, kv->type, &pmix_packed_data, &pmix_packed_data_offset))) {
         OPAL_ERROR_LOG(rc);
@@ -526,8 +540,8 @@ static int s2_put(opal_pmix_scope_t scope,
 
     if (((pmix_packed_data_offset/3)*4) + pmix_packed_encoded_data_offset < pmix_vallen_max) {
         /* this meta-key is still being filled,
-         * nothing to put yet
-         */
+     * nothing to put yet
+     */
         return OPAL_SUCCESS;
     }
 
@@ -574,7 +588,7 @@ static void fencenb(int sd, short args, void *cbdata)
     if (!got_modex_data) {
         got_modex_data = true;
         /* we only need to set locality for each local rank as "not found"
-         * equates to "non-local" */
+     * equates to "non-local" */
         for (i=0; i < s2_nlranks; i++) {
             pname.vpid = s2_lranks[i];
             rc = opal_pmix_base_cache_keys_locally(&s2_pname, OPAL_PMIX_CPUSET,
@@ -585,8 +599,8 @@ static void fencenb(int sd, short args, void *cbdata)
             }
             if (NULL == kp || NULL == kp->data.string) {
                 /* if we share a node, but we don't know anything more, then
-                 * mark us as on the node as this is all we know
-                 */
+         * mark us as on the node as this is all we know
+         */
                 locality = OPAL_PROC_ON_CLUSTER | OPAL_PROC_ON_CU | OPAL_PROC_ON_NODE;
             } else {
                 /* determine relative location on our node */
@@ -612,16 +626,16 @@ static void fencenb(int sd, short args, void *cbdata)
         }
     }
 
-  cleanup:
+cleanup:
     if (NULL != op->opcbfunc) {
-      op->opcbfunc(rc, op->cbdata);
+        op->opcbfunc(rc, op->cbdata);
     }
     OBJ_RELEASE(op);
     return;
 }
 
 static int s2_fencenb(opal_list_t *procs, int collect_data,
-                    opal_pmix_op_cbfunc_t cbfunc, void *cbdata)
+                      opal_pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
     pmi_opcaddy_t *op;
 
@@ -638,9 +652,9 @@ static int s2_fencenb(opal_list_t *procs, int collect_data,
 
 #define S2_WAIT_FOR_COMPLETION(a)               \
     do {                                        \
-        while ((a)) {                           \
-            usleep(10);                         \
-        }                                       \
+    while ((a)) {                           \
+    usleep(10);                         \
+    }                                       \
     } while (0)
 
 struct fence_result {
@@ -736,22 +750,22 @@ static char* pmix_error(int pmix_err)
     char * err_msg;
 
     switch(pmix_err) {
-        case PMI2_FAIL: err_msg = "Operation failed"; break;
-        case PMI2_ERR_INIT: err_msg = "PMI is not initialized"; break;
-        case PMI2_ERR_NOMEM: err_msg = "Input buffer not large enough"; break;
-        case PMI2_ERR_INVALID_ARG: err_msg = "Invalid argument"; break;
-        case PMI2_ERR_INVALID_KEY: err_msg = "Invalid key argument"; break;
-        case PMI2_ERR_INVALID_KEY_LENGTH: err_msg = "Invalid key length argument"; break;
-        case PMI2_ERR_INVALID_VAL: err_msg = "Invalid value argument"; break;
-        case PMI2_ERR_INVALID_VAL_LENGTH: err_msg = "Invalid value length argument"; break;
-        case PMI2_ERR_INVALID_LENGTH: err_msg = "Invalid length argument"; break;
-        case PMI2_ERR_INVALID_NUM_ARGS: err_msg = "Invalid number of arguments"; break;
-        case PMI2_ERR_INVALID_ARGS: err_msg = "Invalid args argument"; break;
-        case PMI2_ERR_INVALID_NUM_PARSED: err_msg = "Invalid num_parsed length argument"; break;
-        case PMI2_ERR_INVALID_KEYVALP: err_msg = "Invalid keyvalp argument"; break;
-        case PMI2_ERR_INVALID_SIZE: err_msg = "Invalid size argument"; break;
-        case PMI2_SUCCESS: err_msg = "Success"; break;
-        default: err_msg = "Unkown error";
+    case PMI2_FAIL: err_msg = "Operation failed"; break;
+    case PMI2_ERR_INIT: err_msg = "PMI is not initialized"; break;
+    case PMI2_ERR_NOMEM: err_msg = "Input buffer not large enough"; break;
+    case PMI2_ERR_INVALID_ARG: err_msg = "Invalid argument"; break;
+    case PMI2_ERR_INVALID_KEY: err_msg = "Invalid key argument"; break;
+    case PMI2_ERR_INVALID_KEY_LENGTH: err_msg = "Invalid key length argument"; break;
+    case PMI2_ERR_INVALID_VAL: err_msg = "Invalid value argument"; break;
+    case PMI2_ERR_INVALID_VAL_LENGTH: err_msg = "Invalid value length argument"; break;
+    case PMI2_ERR_INVALID_LENGTH: err_msg = "Invalid length argument"; break;
+    case PMI2_ERR_INVALID_NUM_ARGS: err_msg = "Invalid number of arguments"; break;
+    case PMI2_ERR_INVALID_ARGS: err_msg = "Invalid args argument"; break;
+    case PMI2_ERR_INVALID_NUM_PARSED: err_msg = "Invalid num_parsed length argument"; break;
+    case PMI2_ERR_INVALID_KEYVALP: err_msg = "Invalid keyvalp argument"; break;
+    case PMI2_ERR_INVALID_SIZE: err_msg = "Invalid size argument"; break;
+    case PMI2_SUCCESS: err_msg = "Success"; break;
+    default: err_msg = "Unkown error";
     }
     return err_msg;
 }
