@@ -14,7 +14,7 @@
  *                         reserved.
  * Copyright (c) 2008-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2010-2016 Los Alamos National Security, LLC.
+ * Copyright (c) 2010-2014 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2014      Hochschule Esslingen.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
@@ -44,9 +44,9 @@
 #include "opal/mca/base/mca_base_var.h"
 #include "opal/runtime/opal_params.h"
 #include "opal/dss/dss.h"
+#include "opal/util/opal_environ.h"
 #include "opal/util/show_help.h"
 #include "opal/util/timings.h"
-#include "opal/util/bit_ops.h"
 
 char *opal_signal_string = NULL;
 char *opal_stacktrace_output_filename = NULL;
@@ -61,6 +61,10 @@ bool opal_timing_overhead = true;
 
 bool opal_built_with_cuda_support = OPAL_INT_TO_BOOL(OPAL_CUDA_SUPPORT);
 bool opal_cuda_support = false;
+bool opal_warn_on_missing_libcuda = true;
+#if OPAL_ENABLE_FT_CR == 1
+bool opal_base_distill_checkpoint_ready = false;
+#endif
 
 /**
  * Globals imported from the OMPI layer.
@@ -69,7 +73,6 @@ int opal_leave_pinned = -1;
 bool opal_leave_pinned_pipeline = false;
 bool opal_abort_print_stack = false;
 int opal_abort_delay = 0;
-unsigned int opal_progress_lp_call_ratio = 8;
 
 static bool opal_register_done = false;
 
@@ -181,6 +184,19 @@ int opal_register_params(void)
     }
 #endif
 
+#if OPAL_ENABLE_FT_CR == 1
+    opal_base_distill_checkpoint_ready = false;
+    ret = mca_base_var_register("opal", "opal", "base", "distill_checkpoint_ready",
+                                "Distill only those components that are Checkpoint Ready",
+                                MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
+                                OPAL_INFO_LVL_8, MCA_BASE_VAR_SCOPE_LOCAL,
+                                &opal_base_distill_checkpoint_ready);
+
+    if (0 > ret) {
+        return ret;
+    }
+#endif
+
     /* RFC1918 defines
        - 10.0.0./8
        - 172.16.0.0/12
@@ -230,13 +246,22 @@ int opal_register_params(void)
         return ret;
     }
 
+    opal_warn_on_missing_libcuda = true;
+    ret = mca_base_var_register ("opal", "opal", NULL, "warn_on_missing_libcuda",
+                                 "Whether to print a message when CUDA support is enabled but libcuda is not found",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, MCA_BASE_VAR_FLAG_SETTABLE,
+                                 OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_ALL_EQ,
+                                 &opal_warn_on_missing_libcuda);
+    if (0 > ret) {
+        return ret;
+    }
+
     /* Leave pinned parameter */
     opal_leave_pinned = -1;
     ret = mca_base_var_register("ompi", "mpi", NULL, "leave_pinned",
-                                "Whether to use the \"leave pinned\" protocol or not.  Enabling this setting can help bandwidth performance when repeatedly sending and receiving large messages with the same buffers over RDMA-based networks (0 = do not use \"leave pinned\" protocol, 1 = use \"leave pinned\" protocol, -1 = allow network to choose at runtime).",
-                                MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
-                                OPAL_INFO_LVL_9,
-                                MCA_BASE_VAR_SCOPE_READONLY,
+                                "Whether to use the \"leave pinned\" protocol or not.  Enabling this setting can help bandwidth performance when repeatedly sending and receiving large messages with the same buffers over RDMA-based networks (false = do not use \"leave pinned\" protocol, true = use \"leave pinned\" protocol, auto = allow network to choose at runtime).",
+                                MCA_BASE_VAR_TYPE_INT, &mca_base_var_enum_auto_bool, 0, 0,
+                                OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                 &opal_leave_pinned);
     mca_base_var_register_synonym(ret, "opal", "opal", NULL, "leave_pinned",
                                   MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
@@ -258,34 +283,6 @@ int opal_register_params(void)
                        true);
     }
 
-#if OPAL_ENABLE_TIMING
-    opal_timing_sync_file = NULL;
-    (void) mca_base_var_register ("opal", "opal", NULL, "timing_sync_file",
-                                  "Clock synchronisation information generated by mpisync tool. You don't need to touch this if you use mpirun_prof tool.",
-                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
-                                  &opal_timing_sync_file);
-    if( opal_timing_clocksync_read(opal_timing_sync_file) ){
-        opal_output(0, "Cannot read file %s containing clock synchronisation information\n", opal_timing_sync_file);
-    }
-
-    opal_timing_output = NULL;
-    (void) mca_base_var_register ("opal", "opal", NULL, "timing_output",
-                                  "The name of output file for timing information. If this parameter is not set then output will be directed into OPAL debug channel.",
-                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
-                                  &opal_timing_output);
-
-    opal_timing_overhead = true;
-    (void) mca_base_var_register ("opal", "opal", NULL, "timing_overhead",
-                                  "Timing framework introduce additional overhead (malloc's mostly)."
-                                  " The time spend in such costly routines is measured and may be accounted"
-                                  " (subtracted from timestamps). 'true' means consider overhead, 'false' - ignore (default: true).",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL,
-                                  &opal_timing_overhead);
-#endif
-
     opal_warn_on_fork = true;
     (void) mca_base_var_register("ompi", "mpi", NULL, "warn_on_fork",
                                  "If nonzero, issue a warning if program forks under conditions that could cause system errors",
@@ -302,26 +299,7 @@ int opal_register_params(void)
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &opal_abort_delay);
     if (0 > ret) {
-	return ret;
-    }
-
-    opal_progress_lp_call_ratio = 8;
-    ret = mca_base_var_register("opal", "opal", NULL, "progress_lp_call_ratio",
-                                "Ratio of calls to high-priority to low-priority progress "
-                                "functions. Higher numbers decrease the frequency of the callback "
-                                "rate. Must be a power of two (default: 8)",
-                                MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0,
-                                OPAL_INFO_LVL_5,
-                                MCA_BASE_VAR_SCOPE_READONLY,
-                                &opal_progress_lp_call_ratio);
-    if (0 > ret) {
-	return ret;
-    }
-
-    if (opal_progress_lp_call_ratio & (opal_progress_lp_call_ratio - 1)) {
-        opal_output(0, "MCA variable progress_lp_call_ratio must be a power of two. value = %u",
-                    opal_progress_lp_call_ratio);
-        return OPAL_ERR_BAD_PARAM;
+        return ret;
     }
 
     opal_abort_print_stack = false;
@@ -342,9 +320,44 @@ int opal_register_params(void)
 #endif
                                  &opal_abort_print_stack);
     if (0 > ret) {
-	return ret;
+        return ret;
     }
 
+    /* register the envar-forwarding params */
+    (void)mca_base_var_register ("opal", "mca", "base", "env_list",
+                                 "Set SHELL env variables",
+                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                 MCA_BASE_VAR_SCOPE_READONLY, &mca_base_env_list);
+
+    mca_base_env_list_sep = MCA_BASE_ENV_LIST_SEP_DEFAULT;
+    (void)mca_base_var_register ("opal", "mca", "base", "env_list_delimiter",
+                                 "Set SHELL env variables delimiter. Default: semicolon ';'",
+                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                 MCA_BASE_VAR_SCOPE_READONLY, &mca_base_env_list_sep);
+
+    /* Set OMPI_MCA_mca_base_env_list variable, it might not be set before
+     * if mca variable was taken from amca conf file. Need to set it
+     * here because mca_base_var_process_env_list is called from schizo_ompi.c
+     * only when this env variable was set.
+     */
+    if (NULL != mca_base_env_list) {
+        char *name = NULL;
+        (void) mca_base_var_env_name ("mca_base_env_list", &name);
+        if (NULL != name) {
+            opal_setenv(name, mca_base_env_list, false, &environ);
+            free(name);
+        }
+    }
+
+    /* Register internal MCA variable mca_base_env_list_internal. It can be set only during
+     * parsing of amca conf file and contains SHELL env variables specified via -x there.
+     * Its format is the same as for mca_base_env_list.
+     */
+    (void)mca_base_var_register ("opal", "mca", "base", "env_list_internal",
+            "Store SHELL env variables from amca conf file",
+            MCA_BASE_VAR_TYPE_STRING, NULL, 0, MCA_BASE_VAR_FLAG_INTERNAL, OPAL_INFO_LVL_3,
+            MCA_BASE_VAR_SCOPE_READONLY, &mca_base_env_list_internal);
+
     /* The ddt engine has a few parameters */
     ret = opal_datatype_register_params();
     if (OPAL_SUCCESS != ret) {
