@@ -53,7 +53,7 @@ static int rte_finalize(void);
 orte_ess_base_module_t orte_ess_tm_module = {
     rte_init,
     rte_finalize,
-    orte_ess_base_app_abort,
+    NULL,
     NULL /* ft_event */
 };
 
@@ -129,14 +129,6 @@ static int rte_finalize(void)
             ORTE_ERROR_LOG(ret);
         }
         return ret;
-    } else {
-        /* otherwise, I must be an application process
-         * use the default procedure to finish
-         */
-        if (ORTE_SUCCESS != (ret = orte_ess_base_app_finalize())) {
-            ORTE_ERROR_LOG(ret);
-            return ret;
-        }
     }
 
     return ORTE_SUCCESS;
